/*
 * *************************************************
 * Copyright (c) 2020 QMTH. All Rights Reserved.
 * Created by Deason on 2020-04-01 18:01:44.
 * *************************************************
 */

package cn.com.qmth.examcloud.exchange.outer.service.impl;

import cn.com.qmth.examcloud.core.oe.admin.api.ExamCourseCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamCourseBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.GetExamCourseReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.GetExamCourseResp;
import cn.com.qmth.examcloud.exchange.outer.service.OutletPaperStructService;
import cn.com.qmth.examcloud.exchange.outer.service.bean.OuterCourseBean;
import cn.com.qmth.examcloud.exchange.outer.service.bean.OuterQuestionBean;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionGroup;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionStructureWrapper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionUnitWrapper;
import cn.com.qmth.examcloud.question.commons.core.question.QuestionType;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.ExtractConfigCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExtractConfigDetailCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExtractConfigPaperCacheBean;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service("outletPaperStructService")
public class OutletPaperStructServiceImpl implements OutletPaperStructService {

    private static final Logger log = LoggerFactory.getLogger(OutletPaperStructServiceImpl.class);

    @Autowired
    private ExamCourseCloudService examCourseCloudService;

    /**
     * 根据考试和课程获取调卷规则下试卷的试题列表
     *
     * @param examId     考试ID
     * @param courseCode 课程代码
     * @return
     */
    @Override
    public List<OuterQuestionBean> getPaperStructQuestions(Long examId, String courseCode) {
        try {
            // 获取题库调卷规则
            ExtractConfigCacheBean extractConfig = CacheHelper.getExtractConfig(examId, courseCode);
            if (extractConfig == null || CollectionUtils.isEmpty(extractConfig.getDetails())) {
                // 调卷规则不存在
                return new ArrayList<>();
            }

            // 默认选取调卷规则中的任意一份试卷
            ExtractConfigDetailCacheBean examPaper = extractConfig.getDetails().get(0);
            log.info(examPaper.getPaperId() + " - " + examPaper.getGroupCode());

            // 获取当前的试卷结构
            ExtractConfigPaperCacheBean extractConfigPaper = CacheHelper.getExtractConfigPaper(examId, courseCode, examPaper.getGroupCode(), examPaper.getPaperId());
            List<DefaultQuestionGroup> questionGroupList = extractConfigPaper.getDefaultPaper().getQuestionGroupList();

            return this.parseSubjectiveQuestions(questionGroupList);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new ArrayList<>();
        }
    }

    private List<OuterQuestionBean> parseSubjectiveQuestions(List<DefaultQuestionGroup> questionGroupList) {
        log.info("parseSubjectiveQuestions start...");

        List<OuterQuestionBean> subjectiveQuestions = new ArrayList<>();

        int order = 0;
        for (int i = 0; i < questionGroupList.size(); i++) {
            DefaultQuestionGroup group = questionGroupList.get(i);

            for (DefaultQuestionStructureWrapper questionWrapper : group.getQuestionWrapperList()) {
                for (DefaultQuestionUnitWrapper questionUnitWrapper : questionWrapper.getQuestionUnitWrapperList()) {
                    int mainNumber = i + 1, subNumber = ++order;

                    if (QuestionType.FILL_UP == questionUnitWrapper.getQuestionType() || QuestionType.ESSAY == questionUnitWrapper.getQuestionType()) {
                        // 只封装主观题信息
                        OuterQuestionBean subjectiveQuestion = new OuterQuestionBean();
                        subjectiveQuestion.setMainNumber(mainNumber);
                        subjectiveQuestion.setMainTitle(String.format("第%s大题", mainNumber));
                        subjectiveQuestion.setSubNumber(subNumber);
                        subjectiveQuestion.setTotalScore(questionUnitWrapper.getQuestionScore());

                        subjectiveQuestions.add(subjectiveQuestion);
                    }
                }
            }
        }

        return subjectiveQuestions;
    }

    /**
     * 获取考试相关联的课程列表
     *
     * @param examId 考试ID
     * @return 列表
     */
    @Override
    public List<OuterCourseBean> getExamCourses(Long examId) {
        try {
            GetExamCourseReq req = new GetExamCourseReq();
            req.setExamId(examId);
            GetExamCourseResp resp = examCourseCloudService.getExamCourses(req);

            List<OuterCourseBean> courses = new ArrayList<>();
            for (ExamCourseBean bean : resp.getList()) {
                OuterCourseBean course = new OuterCourseBean();
                course.setId(bean.getId());
                course.setCode(bean.getCode());
                course.setName(bean.getName());
                courses.add(course);
            }

            return courses;
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new ArrayList<>();
        }
    }

}